<?php

namespace Drupal\views_birthday\Plugin\views\sort;

use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime\Plugin\views\sort\Date;
use Drupal\views_birthday\Plugin\views\query\BirthdayDateSqlInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Basic sort handler for datetime fields.
 *
 * This handler enables granularity, which is the ability to make dates
 * equivalent based upon nearness.
 *
 * @ViewsSort("birthday")
 */
class Birthday extends Date {

  /**
   * The Birthday Date SQL service.
   *
   * @var \Drupal\views_birthday\Plugin\views\query\BirthdayDateSqlInterface
   */
  protected $birthday_date_sql;

  /**
   * Constructs a new Birthday filter handler.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\views_birthday\Plugin\views\query\BirthdayDateSqlInterface $date_sql
   *   The Birthday Date SQL service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, BirthdayDateSqlInterface $date_sql) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->birthday_date_sql = $date_sql;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('views_birthday.date_sql')
    );
  }

  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['granularity'] = ['default' => 'day'];

    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    unset($form['granularity']['#options']['second']);
    unset($form['granularity']['#options']['minute']);
    unset($form['granularity']['#options']['hour']);
  }

  /**
   * {@inheritdoc}
   *
   * Override to account for dates stored as strings.
   */
  public function getDateField() {
    return $this->birthday_date_sql->getNextBirthdayFormula("$this->tableAlias.$this->realField");
  }

  /**
   * {@inheritdoc}
   *
   * Overridden in order to pass in the string date flag.
   */
  public function getDateFormat($format) {
    return $this->query->getDateFormat($this->getDateField(), $format, TRUE);
  }
}
